/*
 Copyright (C) 1996 Mike White
 Permission is granted to any individual or institution to use, copy, or
 redistribute this software so long as all of the original files are included,
 that it is not sold for profit, and that this copyright notice is retained.

*/
/* Windows Info-ZIP Unzip DLL module
 *
 * Author: Mike White
 *
 * Original: 1996
 *
 * This module has the entry points for "unzipping" a zip file.
 */
#include <windows.h>
#define UNZIP_INTERNAL
#include "unzip.h"
#include "version.h"
#include "windll.h"

long ziplen;
uch *outout;

HANDLE hOutBuf;
HANDLE hInBuf;
HANDLE hZipFN;
HANDLE hwildZipFN;
HANDLE hFileName;

HANDLE hInst;               /* current instance */
HMENU  hMenu;               /* main menu handle */

HANDLE hHourGlass;          /* handle to hourglass cursor */
HANDLE hSaveCursor;         /* current cursor handle */
HWND hWndMain;          /* the main window handle. */
HWND hWndList;          /* list box handle */

LPDCL lpDCL;
HANDLE hDCL;
LPUSERFUNCTIONS lpUserFunctions;

/* For displaying status messages and error messages */
int DllMessagePrint(zvoid *pG, uch *buf, ulg size, int flag);
/* For displaying files extracted to the display window */
int DllDisplayPrint(zvoid *pG, uch *buf, ulg size, int flag);

DLLPRNT *lpPrint;

/*  DLL Entry Point */

#ifdef __BORLANDC__
#pragma warn -par
#endif
#if defined WIN32
BOOL WINAPI DllEntryPoint( HINSTANCE hInstance,
                           DWORD fdwRreason,
                           LPVOID plvReserved)
#else
int FAR PASCAL LibMain( HINSTANCE hInstance,
                        WORD wDataSegment,
                        WORD wHeapSize,
                        LPSTR lpszCmdLine )
#endif
{
#ifndef WIN32
/* The startup code for the DLL initializes the local heap(if there is one)
 * with a call to LocalInit which locks the data segment.
 */

if ( wHeapSize != 0 )
   {
   UnlockData( 0 );
   }
#endif
hInst = hInstance;
return 1;   /* Indicate that the DLL was initialized successfully. */
}

int FAR PASCAL WEP ( int bSystemExit )
{
return 1;
}
#ifdef __BORLANDC__
#pragma warn .par
#endif

/* DLL calls */

BOOL WINAPI Unz_Init(__G__ lpUserFunc)
__GDEF
USERFUNCTIONS far * lpUserFunc;
{
lpUserFunctions = lpUserFunc;
lpPrint = lpUserFunc->print;
G.message = DllMessagePrint;
G.sound = lpUserFunc->sound;
G.replace = lpUserFunc->replace;
G.uzp_password = lpUserFunc->password;

if (!lpPrint ||
    !G.sound ||
    !G.replace)
    return FALSE;

return TRUE;
}

jmp_buf dll_error_return;

int WINAPI windll_unzip(int argc, char **FNV, DCL far*C,
   USERFUNCTIONS far *lpUserFunc)
{
int retcode;
CONSTRUCTGLOBALS();

if (!Unz_Init(__G__ lpUserFunc))
   {
   DESTROYGLOBALS();
   return PK_BADERR;
   }

if (C->lpszZipFN == NULL) /* Something has screwed up, we don't have a filename */
   {
   DESTROYGLOBALS();
   return PK_NOZIP;
   }

retcode = setjmp(dll_error_return);
if (retcode)
   {
   DESTROYGLOBALS();
   return PK_BADERR;
   }


lpDCL = C;

Unz_SetOpts(__G__ C);

/* Here is the actual call to "unzip" the files (or whatever else you
 * are doing.)
 */
retcode = Unz_Unzip(__G__ argc, FNV);

DESTROYGLOBALS();
return retcode;
}

int WINAPI Unz_Unzip(__G__ argc, FNV)
__GDEF
int argc;
char **FNV;
{
int retcode;

G.filespecs = argc;
if (stat(G.zipfn, &G.statbuf) || (G.statbuf.st_mode & S_IFMT) == S_IFDIR)
   strcat(G.zipfn, ZSUFX);

if (stat(G.zipfn, &G.statbuf)) {  /* try again */
   char str[PATH_MAX];
   sprintf(str, "error:  can't find zipfile [ %s ]\n", G.zipfn);
   win_fprintf(stdout, lstrlen(str), str);
   return FALSE;
   }
else
   ziplen = G.statbuf.st_size;

if (argc != 0) {
    G.pfnames = FNV;
    G.process_all_files = FALSE;
    }
else
    G.process_all_files = TRUE;       /* for speed */

/*---------------------------------------------------------------------------
    Okey dokey, we have everything we need to get started.  Let's roll.
  ---------------------------------------------------------------------------*/

if ((hInBuf = GlobalAlloc(GMEM_MOVEABLE, INBUFSIZ+4))!=(HGLOBAL)NULL)
   {  /* 4 extra: hold[] */
   G.inbuf = (uch *) GlobalLock(hInBuf);
   WinAssert(G.inbuf);
   }
if ((hOutBuf = GlobalAlloc(GMEM_MOVEABLE, OUTBUFSIZ+1))!=(HGLOBAL)NULL)
   {  /* extra: ASCIIZ */
   G.outbuf = (uch *)GlobalLock(hOutBuf);
   WinAssert(G.outbuf);
   outout = G.outbuf;  /*  point to outbuf */
   }
if ((hFileName = GlobalAlloc(GMEM_MOVEABLE, FILNAMSIZ))!=0)
   {
   G.filename = GlobalLock(hFileName);
   WinAssert(G.filename);
   }

if ((G.inbuf == NULL) || (G.outbuf == NULL) || (outout == NULL) ||
   (G.zipfn == NULL) || (G.filename == NULL))
   {
   return FALSE;
   }

G.hold = &G.inbuf[INBUFSIZ];   /* to check for boundary-spanning signatures */
G.key = (char *)NULL;
retcode = process_zipfiles(__G);
FreeDllMem(__G);
return retcode;
}

int win_fprintf(FILE *file, unsigned int size, char far *buffer)
{
if ((file != stderr) && (file != stdout))
   {
   return write(fileno(file),(char far *)(buffer),size);
   }
return lpPrint(buffer, size);
}

/**********************************
 * Function DllMessagePrint()     *
 *                                *
 * Send messages to status window *
 **********************************/
#ifdef __BORLANDC__
#pragma argsused
#endif
int DllMessagePrint(pG, buf, size, flag)
    zvoid *pG;      /* globals struct:  always passed */
    uch *buf;       /* preformatted string to be printed */
    ulg size;       /* length of string (may include nulls) */
    int flag;       /* flag bits */
{
return lpPrint((char far *)buf, size);
}

/********************************
 * Function DllDisplayPrint()   *
 *                              *
 * Send files to display window *
 ********************************/
#ifdef __BORLANDC__
#pragma argsused
#endif
int DllDisplayPrint(pG, buf, size, flag)
    zvoid *pG;      /* globals struct:  always passed */
    uch *buf;       /* preformatted string to be printed */
    ulg size;       /* length of string (may include nulls) */
    int flag;       /* flag bits */
{
return lpPrint((char far *)buf, size);
}


int WINAPI unzipVB(int argc, char **FNV, DCL far *C,
   VBUSERFUNCTIONS far *lpUF)
{
int retcode;
HANDLE hUF;
LPUSERFUNCTIONS lpUserFunc;
void * lpSound;
void * lpPassword;
void * lpMessage;
void * lpReplace;
void * lpPrintVB;
CONSTRUCTGLOBALS();

hUF = GlobalAlloc( GPTR, (DWORD)sizeof(VBUSERFUNCTIONS));
if (!hUF)
   {
   DESTROYGLOBALS();
   return PK_MEM;
   }
lpUserFunc = (LPUSERFUNCTIONS)GlobalLock(hUF);
if (!lpUserFunc)
   {
   GlobalFree(hUF);
   DESTROYGLOBALS();
   return PK_MEM;
   }
lpPrintVB = GetProcAddress(lpUF->hInstance,
   lpUF->print);
lpUserFunc->print = lpPrintVB;   
if (!lpUserFunc->print)
   {
   GlobalUnlock(hUF);
   GlobalFree(hUF);
   DESTROYGLOBALS();
   return PK_BADERR;
   }
lpSound = GetProcAddress(lpUF->hInstance,lpUF->sound);
lpUserFunc->sound = lpSound;
if (!lpUserFunc->sound)
   {
   GlobalUnlock(hUF);
   GlobalFree(hUF);
   DESTROYGLOBALS();
   return PK_BADERR;
   }
(int far *WINAPI)lpReplace = (int far *WINAPI)GetProcAddress(lpUF->hInstance,lpUF->replace);
lpUserFunc->replace = lpReplace;
if (!lpUserFunc->replace)
   {
   GlobalUnlock(hUF);
   GlobalFree(hUF);
   DESTROYGLOBALS();
   return PK_BADERR;
   }
(char * WINAPI)lpPassword = (char * WINAPI)GetProcAddress(lpUF->hInstance,
   lpUF->password);
lpUserFunc->password = lpPassword;
if (!lpUserFunc->password)
   {
   GlobalUnlock(hUF);
   GlobalFree(hUF);
   DESTROYGLOBALS();
   return PK_BADERR;
   }
lpMessage = GetProcAddress(lpUF->hInstance,
   lpUF->SendApplicationMessage);
lpUserFunc->SendApplicationMessage = lpMessage;
if (!lpUserFunc->SendApplicationMessage)
   {
   GlobalUnlock(hUF);
   GlobalFree(hUF);
   DESTROYGLOBALS();
   return PK_BADERR;
   }
lpUserFunc->cchComment = lpUF->cchComment;
lpUserFunc->TotalSizeComp = lpUF->TotalSizeComp;
lpUserFunc->TotalSize = lpUF->TotalSize;
lpUserFunc->CompFactor = lpUF->CompFactor;
lpUserFunc->NumMembers = lpUF->NumMembers;

if (!Unz_Init(__G__ lpUserFunc))
   {
   GlobalUnlock(hUF);
   GlobalFree(hUF);
   DESTROYGLOBALS();
   return PK_BADERR;
   }

if (C->lpszZipFN == NULL)
   {
   GlobalUnlock(hUF);
   GlobalFree(hUF);
   DESTROYGLOBALS();
   return PK_NOZIP;
   }

retcode = setjmp(dll_error_return);
if (retcode)
   {
   GlobalUnlock(hUF);
   GlobalFree(hUF);
   DESTROYGLOBALS();
   return PK_BADERR;
   }


lpDCL = C;

Unz_SetOpts(__G__ C);

retcode = Unz_Unzip(__G__ argc, FNV);

GlobalUnlock(hUF);
GlobalFree(hUF);
DESTROYGLOBALS();
return retcode;
}

