/*
 Copyright (C) 1996 Mike White
 Permission is granted to any individual or institution to use, copy, or
 redistribute this software so long as all of the original files are included,
 that it is not sold for profit, and that this copyright notice is retained.

*/

#include <windows.h>
#include <string.h>
#include <stdio.h>
#include "wiz.h"
#ifdef __BORLANDC__
#include <dir.h>
#else
#include <direct.h>
#endif
#include "helpids.h"

int cSelLBItems;
char **pszIndex;
char *sz;
char baseDir[PATH_MAX], tempName[PATH_MAX];
LPSTR szFileList;
HANDLE hFileList;
char szPattern[PATH_MAX], szLastPat[PATH_MAX];


/****************************************************************************

    FUNCTION: GetFilesProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes messages for "Get Files For Archive" dialog box

    MESSAGES:

    WM_INITDIALOG - initialize dialog box
    WM_COMMAND    - Input received

****************************************************************************/
#ifdef __BORLANDC__
#pragma warn -par
#pragma warn -aus
#endif
BOOL WINAPI
GetFilesProc(HWND hwndDlg, WORD wMessage, WPARAM wParam, LPARAM lParam)
{
char szFName[PATH_MAX];
int __far * __far *ppnSelItems, numFiles;
int i, far *pnSelItems;
HGLOBAL pHND;
HWND hTemp;

   switch (wMessage) {
   case WM_INITDIALOG:
      /* This hides the actual OK button associated with the file open
         dialog. We basically want to disable this button, as pressing
         enter will otherwise cause the OK button to be pressed, and the
         dialog to terminate.
       */
      hTemp = GetDlgItem(hwndDlg, IDOK);
      WinAssert(hTemp);
      EnableWindow(hTemp, FALSE);
      ShowWindow(hTemp, SW_HIDE);

      lpZCL->szRootDir[0] = '\0';
      lstrcpy(szLastPat, "*.*");
      numFiles = 0;
      lpZCL->argc = 0;
      CenterDialog(GetParent(hwndDlg), hwndDlg); /* center on parent */
      break;
   case WM_COMMAND:
      switch (LOWORD(wParam)) {
         case IDC_TEMP_PREFERENCES:
            {
#ifndef WIN32
            FARPROC lpfnPrefs;
            lpfnPrefs = MakeProcInstance((FARPROC)ZipPreferencesProc, hInst);
            DialogBox(hInst, "ZIP_PREFS", hwndDlg, lpfnPrefs);
            FreeProcInstance(lpfnPrefs);
#else
            DialogBox(hInst, "ZIP_PREFS", hwndDlg, (DLGPROC)ZipPreferencesProc);
#endif
            }
            break;
         case IDOK:
            if ((HWND)lParam != NULL)
               {
               GetWindowText(GetDlgItem(hwndDlg, edt1), szLastPat, PATH_MAX);
               break;
               }
         case IDC_ADD:
            {
            cSelLBItems = (int)SendMessage(GetDlgItem(hwndDlg, lst1),
               LB_GETSELCOUNT, 0, 0L);
            if (cSelLBItems == (int)SendMessage(GetDlgItem(hwndDlg, lst1),
               LB_GETCOUNT, 0, 0L))
               {
               getcwd(baseDir, PATH_MAX);
               lstrcat(baseDir, "\\");
               lstrcat(baseDir, szLastPat);
               SendMessage(GetDlgItem(hwndDlg, IDC_FILE_LIST),
                  LB_INSERTSTRING, 0, (LONG) ((LPSTR) baseDir));
               return TRUE;
               }
            if (cSelLBItems)
               {
               pHND = GlobalAlloc(GMEM_FIXED, cSelLBItems * sizeof(int));
               WinAssert(pHND);

               pnSelItems = (int __far *)GlobalLock(pHND);
               ppnSelItems = &pnSelItems;
               if ( !*ppnSelItems )
                  {
                  GlobalFree(pHND);
                  return -1;
                  }

               /* Get list of selected items. Return value is number of
                  selected items */
               if (SendMessage(GetDlgItem(hwndDlg, lst1), LB_GETSELITEMS,
                  cSelLBItems, (LONG)*ppnSelItems) != cSelLBItems)
                  {
                  GlobalUnlock(pHND);
                  GlobalFree(pHND);
                  return -1;
                  }

               getcwd(baseDir, PATH_MAX);
               lstrcat(baseDir, "\\");
               for (i = cSelLBItems - 1; i >= 0; i--)
                  {
                  SendMessage(GetDlgItem(hwndDlg, lst1), LB_GETTEXT,
                     pnSelItems[i],(LPARAM) (LPCSTR) szFName);
                  lstrcpy(tempName, baseDir);
                  lstrcat(tempName, szFName);
                  SendMessage(GetDlgItem(hwndDlg, IDC_FILE_LIST),
                     LB_INSERTSTRING, 0, (LONG)(LPSTR) tempName);
                  numFiles++;
                  }
               GlobalUnlock(pHND);
               GlobalFree(pHND);
               }
            SendMessage(GetDlgItem(hwndDlg, lst1) , LB_SELITEMRANGE,
               (WPARAM)FALSE,
                MAKELONG(0,-1));
            SetWindowText(GetDlgItem(hwndDlg, edt1), szLastPat);
            }
            return TRUE;
         case IDC_DELETE:
            {
            cSelLBItems = (int)SendMessage(GetDlgItem(hwndDlg, IDC_FILE_LIST),
               LB_GETSELCOUNT, 0, 0L);

            if (cSelLBItems)
               {
               pHND = GlobalAlloc(GMEM_FIXED, cSelLBItems * sizeof(int));
               WinAssert(pHND);

               pnSelItems = (int __far *)GlobalLock(pHND);
               ppnSelItems = &pnSelItems;
               if ( !*ppnSelItems )
                  {
                  GlobalFree(pHND);
                  return -1;
                  }

               /* Get list of selected items. */
               if (SendMessage(GetDlgItem(hwndDlg, IDC_FILE_LIST), LB_GETSELITEMS,
                  cSelLBItems, (LONG)*ppnSelItems) != cSelLBItems)
                  {
                  GlobalUnlock(pHND);
                  GlobalFree(pHND);
                  return -1;
                  }

               for (i = cSelLBItems - 1; i >= 0; i--)
                  {
                  SendMessage(GetDlgItem(hwndDlg, IDC_FILE_LIST), LB_DELETESTRING,
                     pnSelItems[i],0);
                  numFiles--;
                  }
               GlobalUnlock(pHND);
               GlobalFree(pHND);
               }
            }
            break;
         case IDC_SET_ROOT:
            getcwd(lpZCL->szRootDir, PATH_MAX);
            SetWindowText(GetDlgItem(hwndDlg, IDC_ROOT_DIR), lpZCL->szRootDir);
            break;
         case IDC_FREE_ROOT:
            lpZCL->szRootDir[0] = '\0';
            SetWindowText(GetDlgItem(hwndDlg, IDC_ROOT_DIR), lpZCL->szRootDir);
            break;
         case IDC_INCLUDE:
            /* We don't want to get what is selected in the file box by
               mistake.
             */
            if (SendMessage(GetDlgItem(hwndDlg, lst1),
                  LB_GETSELCOUNT, 0, 0L) > 0)
               break;
            {
            getcwd(baseDir, PATH_MAX);
            lstrcat(baseDir, "\\");
            GetWindowText(GetDlgItem(hwndDlg, edt1), tempName, PATH_MAX);
            lstrcat(baseDir, tempName);
            SendMessage(GetDlgItem(hwndDlg, IDC_FILE_LIST),
               LB_INSERTSTRING, 0, (LONG) ((LPSTR) baseDir));
            }
            break;
         case IDC_RECURSE:
            /* We don't want to get what is selected in the file box by
               mistake.
             */
            if (SendMessage(GetDlgItem(hwndDlg, lst1),
                  LB_GETSELCOUNT, 0, 0L) > 0)
               break;
            {
            char *p, drive[2];
            getcwd(baseDir, PATH_MAX);
            lstrcat(baseDir, "\\");
            GetWindowText(GetDlgItem(hwndDlg, edt1), tempName, PATH_MAX);
            lstrcat(baseDir, tempName);
            drive[0] = baseDir[0];
            drive[1] = '\0';
            p = &baseDir[2];
            FindFile(drive, p, hwndDlg);
            }
            break;
         case IDC_SELECT_ALL:
         case IDC_DESELECT_ALL:
            {
            WORD cZippedFiles;
            cZippedFiles = (WORD)SendMessage(GetDlgItem(hwndDlg, lst1),
               LB_GETCOUNT, 0, 0L);
            SendMessage(GetDlgItem(hwndDlg, lst1) , LB_SELITEMRANGE,
               (WPARAM)(LOWORD(wParam) == IDC_DESELECT_ALL ? FALSE : TRUE),
               MAKELONG(0, (cZippedFiles-1)));
            }
            SetWindowText(GetDlgItem(hwndDlg, edt1), szLastPat);
            break;
         case IDCANCEL:
            lpZCL->lpszZipFN[0] = '\0';
            lpumb->szZipFileName[0] = '\0';
            break;
         case IDC_DONE:
            {
            int cc;
            char **index;

            SendMessage(GetDlgItem(hwndDlg, IDC_FILE_LIST) , LB_SELITEMRANGE,
               (WPARAM)TRUE,
                MAKELONG(0, (numFiles-1)));
            cSelLBItems = (int)SendMessage(GetDlgItem(hwndDlg, IDC_FILE_LIST),
               LB_GETSELCOUNT, 0, 0L);

            if (cSelLBItems < 1)
               {
               if (!lpZCL->fRepair)
                  {
                  lpZCL->lpszZipFN[0] = '\0';
                  lpumb->szZipFileName[0] = '\0';
                  }
               lpZCL->argc = 0;
               goto pau;
               }

            lpZCL->argc = cSelLBItems; /* keep count to transfer to dll */

            hFileList = GlobalAlloc( GPTR, FILENAME_BUF_SIZE);
            if ( hFileList )
               {
               szFileList = (char far *)GlobalLock(hFileList);
               }
            else
               {
               MessageBox(hWndMain, "Try archiving fewer files at a time",
                  "Insufficient Memory", MB_OK |MB_SYSTEMMODAL|
                  MB_ICONEXCLAMATION);
               lpZCL->lpszZipFN[0] = '\0';
               lpumb->szZipFileName[0] = '\0';
               goto pau;
               }

            if (cSelLBItems)
               {
               index = (char **)szFileList;
               cc = (sizeof(char *) * lpZCL->argc);
               sz = szFileList + cc;

               for (i = 0; i < lpZCL->argc; i++)
                  {
                  cc = (int)SendMessage(GetDlgItem(hwndDlg, IDC_FILE_LIST),
                     LB_GETTEXT, i,(LPARAM) (LPCSTR) szFName);
                  lstrcpy(sz, szFName);
                  index[i] = sz;
                  sz += (cc + 1);
                  }
               }
            }
            SendMessage(GetDlgItem(hwndDlg, IDC_FILE_LIST) , LB_SELITEMRANGE,
               (WPARAM)FALSE,
                MAKELONG(0,-1));
            SendMessage(GetDlgItem(hwndDlg, lst1) , LB_SELITEMRANGE,
               (WPARAM)FALSE,
                MAKELONG(0,-1));
pau:
            EndDialog(hwndDlg, TRUE);
            break;
         }
      default:
         break;
   }
   return FALSE;
}
#ifdef __BORLANDC__
#pragma warn .aus
#pragma warn .par
#endif

